@@ -25,8 +25,10 @@ Configuration
 
 from ...utils.importlib import require_modules
 
+
 with require_modules(["algoliasearch", "algoliasearch.version"]) as missing_modules:
     if not missing_modules:
-        from .patch import patch, unpatch
+        from .patch import patch
+        from .patch import unpatch
 
         __all__ = ["patch", "unpatch"]
