@@ -3,27 +3,32 @@ from collections import defaultdict
 from json import loads
 import logging
 import sys
-import threading
 from typing import List
 from typing import Optional
 from typing import TYPE_CHECKING
+from typing import TextIO
+
+import six
+import tenacity
 
 import ddtrace
-from ddtrace.vendor import six
+from ddtrace.vendor.dogstatsd import DogStatsd
 
 from . import agent
-from .. import _worker
-from .. import compat
-from ..compat import httplib
+from . import compat
+from . import periodic
+from . import service
 from ..constants import KEEP_SPANS_RATE_KEY
-from ..encoding import Encoder
-from ..encoding import JSONEncoderV2
 from ..sampler import BasePrioritySampler
+from ..sampler import BaseSampler
+from ..utils.formats import get_env
 from ..utils.time import StopWatch
 from .agent import get_connection
 from .buffer import BufferFull
 from .buffer import BufferItemTooLarge
 from .buffer import TraceBuffer
+from .encoding import Encoder
+from .encoding import JSONEncoderV2
 from .logger import get_logger
 from .runtime import container
 from .sma import SimpleMovingAverage
@@ -43,6 +48,29 @@ LOG_ERR_INTERVAL = 60
 # to 10 buckets of 1s duration.
 DEFAULT_SMA_WINDOW = 10
 
+DEFAULT_BUFFER_SIZE = 8 * 1000000  # 8mb
+DEFAULT_MAX_PAYLOAD_SIZE = 8 * 1000000  # 8mb
+DEFAULT_PROCESSING_INTERVAL = 1.0
+
+
+def get_writer_buffer_size():
+    # type: () -> int
+    return int(get_env("trace", "writer_buffer_size_bytes", default=DEFAULT_BUFFER_SIZE))  # type: ignore[arg-type]
+
+
+def get_writer_max_payload_size():
+    # type: () -> int
+    return int(
+        get_env("trace", "writer_max_payload_size_bytes", default=DEFAULT_MAX_PAYLOAD_SIZE)  # type: ignore[arg-type]
+    )
+
+
+def get_writer_interval_seconds():
+    # type: () -> float
+    return float(
+        get_env("trace", "writer_interval_seconds", default=DEFAULT_PROCESSING_INTERVAL)  # type: ignore[arg-type]
+    )
+
 
 def _human_size(nbytes):
     """Return a human-readable size."""
@@ -106,7 +134,10 @@ class Response(object):
                 # This typically happens when using a priority-sampling enabled
                 # library with an outdated agent. It still works, but priority sampling
                 # will probably send too many traces, so the next step is to upgrade agent.
-                log.debug("Cannot parse Datadog Agent response, please make sure your Datadog Agent is up to date")
+                log.debug(
+                    "Cannot parse Datadog Agent response. "
+                    "This occurs because Datadog agent is out of date or DATADOG_PRIORITY_SAMPLING=false is set"
+                )
                 return
 
             return loads(body)
@@ -141,7 +172,13 @@ class TraceWriter(six.with_metaclass(abc.ABCMeta)):
 
 
 class LogWriter(TraceWriter):
-    def __init__(self, out=sys.stdout, sampler=None, priority_sampler=None):
+    def __init__(
+        self,
+        out=sys.stdout,  # type: TextIO
+        sampler=None,  # type: Optional[BaseSampler]
+        priority_sampler=None,  # type: Optional[BasePrioritySampler]
+    ):
+        # type: (...) -> None
         self._sampler = sampler
         self._priority_sampler = priority_sampler
         self.encoder = JSONEncoderV2()
@@ -171,7 +208,7 @@ class LogWriter(TraceWriter):
         self.out.flush()
 
 
-class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
+class AgentWriter(periodic.PeriodicService, TraceWriter):
     """Writer to the Datadog Agent.
 
     The Datadog Agent supports (at the time of writing this) receiving trace
@@ -180,22 +217,25 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
     should be in the same trace.
     """
 
+    RETRY_ATTEMPTS = 3
+
     def __init__(
         self,
-        agent_url,
-        sampler=None,
-        priority_sampler=None,
-        processing_interval=1,
+        agent_url,  # type: str
+        sampler=None,  # type: Optional[BaseSampler]
+        priority_sampler=None,  # type: Optional[BasePrioritySampler]
+        processing_interval=get_writer_interval_seconds(),  # type: float
         # Match the payload size since there is no functionality
         # to flush dynamically.
-        buffer_size=8 * 1000000,
-        max_payload_size=8 * 1000000,
-        timeout=agent.DEFAULT_TIMEOUT,
-        dogstatsd=None,
-        report_metrics=False,
-        sync_mode=False,
+        buffer_size=get_writer_buffer_size(),  # type: int
+        max_payload_size=get_writer_max_payload_size(),  # type: int
+        timeout=agent.get_trace_agent_timeout(),  # type: float
+        dogstatsd=None,  # type: Optional[DogStatsd]
+        report_metrics=False,  # type: bool
+        sync_mode=False,  # type: bool
     ):
-        super(AgentWriter, self).__init__(interval=processing_interval, name=self.__class__.__name__)
+        # type: (...) -> None
+        super(AgentWriter, self).__init__(interval=processing_interval)
         self.agent_url = agent_url
         self._buffer_size = buffer_size
         self._max_payload_size = max_payload_size
@@ -211,9 +251,9 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
         self._timeout = timeout
 
         if priority_sampler is not None:
-            self._endpoint = "/v0.4/traces"
+            self._endpoint = "v0.4/traces"
         else:
-            self._endpoint = "/v0.3/traces"
+            self._endpoint = "v0.3/traces"
 
         self._container_info = container.get_container_info()
         if self._container_info and self._container_info.container_id:
@@ -226,12 +266,20 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
         self._encoder = Encoder()
         self._headers.update({"Content-Type": self._encoder.content_type})
 
-        self._started_lock = threading.Lock()
         self.dogstatsd = dogstatsd
         self._report_metrics = report_metrics
         self._metrics_reset()
         self._drop_sma = SimpleMovingAverage(DEFAULT_SMA_WINDOW)
         self._sync_mode = sync_mode
+        self._retry_upload = tenacity.Retrying(
+            # Retry RETRY_ATTEMPTS times within the first half of the processing
+            # interval, using a Fibonacci policy with jitter
+            wait=tenacity.wait_random_exponential(
+                multiplier=0.618 * self.interval / (1.618 ** self.RETRY_ATTEMPTS) / 2, exp_base=1.618
+            ),
+            stop=tenacity.stop_after_attempt(self.RETRY_ATTEMPTS),
+            retry=tenacity.retry_if_exception_type((compat.httplib.HTTPException, OSError, IOError)),
+        )
 
     def _metrics_dist(self, name, count=1, tags=None):
         self._metrics[name]["count"] += count
@@ -280,7 +328,7 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
                 conn.request("PUT", self._endpoint, data, headers)
                 resp = compat.get_connection_response(conn)
                 t = sw.elapsed()
-                if t >= self._thread.interval:
+                if t >= self.interval:
                     log_level = logging.WARNING
                 else:
                     log_level = logging.DEBUG
@@ -290,8 +338,8 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
                 conn.close()
 
     def _downgrade(self, payload, response):
-        if self._endpoint == "/v0.4/traces":
-            self._endpoint = "/v0.3/traces"
+        if self._endpoint == "v0.4/traces":
+            self._endpoint = "v0.3/traces"
             return payload
         raise ValueError
 
@@ -347,16 +395,16 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
 
     def write(self, spans=None):
         # type: (Optional[List[Span]]) -> None
-        # Start the AgentWriter on first write.
-        # Starting it earlier might be an issue with gevent, see:
-        # https://github.com/DataDog/dd-trace-py/issues/1192
         if spans is None:
             return
 
-        if self.started is False and self._sync_mode is False:
-            with self._started_lock:
-                if self.started is False:
+        if self._sync_mode is False:
+            # Start the AgentWriter on first write.
+            try:
+                if self.status != service.ServiceStatus.RUNNING:
                     self.start()
+            except service.ServiceStatusError:
+                pass
 
         self._metrics_dist("writer.accepted.traces")
         self._set_keep_rate(spans)
@@ -394,6 +442,7 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
                     self.flush_queue()
 
     def flush_queue(self, raise_exc=False):
+        # type: (bool) -> None
         enc_traces = self._buffer.get()
         try:
             if not enc_traces:
@@ -401,17 +450,17 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
 
             encoded = self._encoder.join_encoded(enc_traces)
             try:
-                self._send_payload(encoded, len(enc_traces))
-            except (httplib.HTTPException, OSError, IOError):
+                self._retry_upload(self._send_payload, encoded, len(enc_traces))
+            except tenacity.RetryError as e:
                 self._metrics_dist("http.errors", tags=["type:err"])
                 self._metrics_dist("http.dropped.bytes", len(encoded))
                 self._metrics_dist("http.dropped.traces", len(enc_traces))
                 if raise_exc:
-                    raise
+                    e.reraise()
                 else:
                     log.error("failed to send traces to Datadog Agent at %s", self.agent_url, exc_info=True)
             finally:
-                if self._report_metrics:
+                if self._report_metrics and self.dogstatsd:
                     # Note that we cannot use the batching functionality of dogstatsd because
                     # it's not thread-safe.
                     # https://github.com/DataDog/datadogpy/issues/439
@@ -424,13 +473,16 @@ class AgentWriter(_worker.PeriodicWorkerThread, TraceWriter):
             self._set_drop_rate()
             self._metrics_reset()
 
-    def run_periodic(self):
+    def periodic(self):
         self.flush_queue(raise_exc=False)
 
-    def stop(self, timeout=None):
-        # type: (Optional[float]) -> None
-        if self.is_alive():
-            super(AgentWriter, self).stop()
-            self.join(timeout=timeout)
+    def _stop_service(  # type: ignore[override]
+        self,
+        timeout=None,  # type: Optional[float]
+    ):
+        # type: (...) -> None
+        # FIXME: don't join() on stop(), let the caller handle this
+        super(AgentWriter, self)._stop_service()
+        self.join(timeout=timeout)
 
-    on_shutdown = run_periodic
+    on_shutdown = periodic
